<?php
/**
 * Plugin Name:	WP TimeZone
 * Plugin URI:	http://wordpress.org/plugins/wp-timezone/
 * Description:	Automatically changes the WordPress timezone settings based on CET and CEST.
 * Version: 	1.1
 * Author: 	 	Ezra Verheijen
 * Author URI:  http://www.qualityinternetsolutions.nl/
 * License: 	GPL v3
 */

/*	Copyright (c) 2013, Ezra Verheijen - info@qualityinternetsolutions.nl
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*	If this plugin needs to be installed on a multisite, it can also be
	placed in 'mu-plugins' (http://codex.wordpress.org/Must_Use_Plugins). */

/**
 * WP TimeZone init class
 *
 * @since 1.1
 */
class WP_Timezone {
	
	/**
	 * Constructor
	 *
	 * @since 1.1
	 */
	function __construct() {
		
		add_action( 'init', array( $this, 'set_utc_offset' ) );
		add_action( 'admin_head', array( $this, 'change_settings_screen' ) );
		
		add_filter( 'plugin_row_meta', array( $this, 'wpt_filter_plugin_meta' ), 10, 2 );
		
	}
	
	/**
	 * Set UTC offset based on CET/CEST
	 *
	 * Runs after WordPress has finished loading
	 * but before any headers are sent.
	 * 
	 * @since 1.0
	 */
	function set_utc_offset() {
		
		/* WordPress default sets to UTC timezone in wp-settings.php */
		
		if ( function_exists( 'date_default_timezone_set' ) )
			date_default_timezone_set( 'Europe/Amsterdam' ); // choose one, it's all the same
		
		$timezone = date( 'T' );
		
		/* Now we have the local timezone stored in $timezone,
		so lets restore WordPress setting as in wp-settings.php */
		
		if ( function_exists( 'date_default_timezone_set' ) )
			date_default_timezone_set( 'UTC' );
		
		/** Get current offset */
		$current_offset = get_option( 'gmt_offset' );
		
		/** Set new offset */
		$offset = 1; // Central European Time (UTC+1)
		
		if ( 'CEST' == $timezone ) // Central European Summer Time
			$offset = 2;
		
		// Update offset in database if new offset is different than current offset
		if ( $offset != $current_offset )
			update_option( 'gmt_offset', $offset );
		
		/**
		 * Set default timezone to show/handle on admin config page
		 * to avoid confusion and avoid user interference.
		 * Leave empty so WordPress can automatically
		 * fill in the offset being used.
		 */
		$timezone_string = '';
		
		/** Check if a user messed with the settings */
		$current_string = get_option( 'timezone_string' );
		
		// If so, set back to default
		if ( $timezone_string != $current_string )
			update_option( 'timezone_string', $timezone_string );
		
	}
	
	/**
	 * Change the Settings->General Screen to be more clever
	 *
	 * Note: I wish there was a nicer way to do this,
	 * but the admin pages lack the proper hooks for it.
	 * Until it gets them, this works.
	 * 
	 * @return null Returns early if not on the options-general page
	 * 
	 * @since 1.0
	 */
	 function change_settings_screen() {
		
		global $pagenow;
		
		if ( 'options-general.php' != $pagenow )
			return;
		
		// All looks good, turn on the output buffer and attach our callback
		ob_start( array( $this, 'replace_text' ) );
		
	}
	
	/**
	 * Put a better comment on the Settings->General page
	 *
	 * @param $buffer The output buffer holding the page data
	 * 
	 * @return $final The page data with the replaced text
	 * 
	 * @since 1.0
	 */
	function replace_text( $buffer ) {
		
		/** Convert all applicable characters to HTML entities */
		$converted = htmlentities( $buffer, ENT_QUOTES, 'UTF-8' );
		
		$search = array(
			'Kies een plaats in dezelfde tijdzone als waarin je zit.', // Dutch
			'Choose a city in the same timezone as you.',              // English
			'W&auml;hle eine Stadt, die in deiner Zeitzone liegt.',    // German
		);
		$replace = array(
			'Tijdzones worden autmatisch beheerd door de WP TimeZone plugin.',    // Dutch
			'Timezones are automatically managed by the WP TimeZone plugin.',     // English
			'Zeitzonen werden automatisch verwaltet bei der WP TimeZone Plugin.', // German
		);
		
		$new_string = str_replace( $search, $replace, $converted );
		
		/** Convert all HTML entities back to their applicable characters */
		$final = html_entity_decode( $new_string, ENT_QUOTES, 'UTF-8' );
		
		return $final;
		
		// Clean output buffer and turn off output buffering
		ob_end_clean();
		
	}
	
	/**
	 * Add a "Donate" link to the plugin meta info
	 * 
	 * @param $links The existing links
	 * @param $file plugin basename
	 * 
	 * @return $links
	 * 
	 * @since 1.1
	 */
	function wpt_filter_plugin_meta( $links, $file ) {
		
		if ( plugin_basename( __FILE__ ) == $file ) {
			
			$lang = get_locale();
			
			$donate = 'Donate';
			
			if ( 'nl_NL' == $lang )
				$donate = 'Doneer';
			
			if ( 'de_DE' == $lang )
				$donate = 'Spenden';
			
			return array_merge(
					$links,
					array( sprintf( '<a href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=3H3RWC5JULGJ4" target="_blank">%s</a>', __( $donate ) ) )
			);
			
		}
		
		return $links;
		
	}
	
}

$wp_timezone = new WP_Timezone();
